@@ -30,7 +30,9 @@ want_syslog=0
 syslog_tag=
 user='@MYSQLD_USER@'
 pid_file=
+pid_file_append=
 err_log=
+err_log_append=
 timestamp_format=UTC
 
 syslog_tag_mysqld=mysqld
@@ -213,6 +215,10 @@ eval_log_error () {
 
   #echo "Running mysqld: [$cmd]"
   eval "$cmd"
+  ret=$?
+  if [ $ret > 0 ]; then
+    exit $ret
+  fi
 }
 
 shell_quote_string() {
@@ -490,8 +496,8 @@ else
   # layout is used
 
   cd "`dirname $0`"
-  if [ -h "$0" ] ; then
-    realpath="`ls -l  "$0" | awk '{print $NF}'`"
+  if [ -h "`dirname $0`" -o -h "$0" ] ; then
+    realpath=$(readlink -f "$0")
     cd "`dirname "$realpath"`"
   fi
   cd ..
@@ -667,15 +673,17 @@ then
       err_log="$err_log".err
     fi
 
+    err_log_append="$err_log"
     case "$err_log" in
       /* ) ;;
       * ) err_log="$DATADIR/$err_log" ;;
     esac
   else
     err_log=$DATADIR/`@HOSTNAME@`.err
+    err_log_append=`@HOSTNAME@`.err
   fi
 
-  append_arg_to_args "--log-error=$err_log"
+  append_arg_to_args "--log-error=$err_log_append"
 
   if [ $want_syslog -eq 1 ]
   then
@@ -773,13 +781,15 @@ fi
 if test -z "$pid_file"
 then
   pid_file="$DATADIR/`@HOSTNAME@`.pid"
+  pid_file_append="`@HOSTNAME@`.pid"
 else
+  pid_file_append="$pid_file"
   case "$pid_file" in
     /* ) ;;
     * )  pid_file="$DATADIR/$pid_file" ;;
   esac
 fi
-append_arg_to_args "--pid-file=$pid_file"
+append_arg_to_args "--pid-file=$pid_file_append"
 
 if test -n "$mysql_unix_port"
 then
