@@ -1,4 +1,4 @@
-/* Copyright (c) 2005, 2016, Oracle and/or its affiliates. All rights reserved.
+/* Copyright (c) 2005, 2017, Oracle and/or its affiliates. All rights reserved.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -504,10 +504,11 @@ public:
   int inc_group_relay_log_pos(ulonglong log_pos,
                               bool need_data_lock);
 
-  int wait_for_pos(THD* thd, String* log_name, longlong log_pos, 
-		   longlong timeout);
-  int wait_for_gtid_set(THD* thd, String* gtid, longlong timeout);
-  int wait_for_gtid_set(THD* thd, const Gtid_set* wait_gtid_set, longlong timeout);
+  int wait_for_pos(THD* thd, String* log_name, longlong log_pos,
+                   double timeout);
+  int wait_for_gtid_set(THD* thd, String* gtid, double timeout);
+  int wait_for_gtid_set(THD* thd, const Gtid_set* wait_gtid_set,
+                        double timeout);
 
   void close_temporary_tables();
 
@@ -840,7 +841,8 @@ public:
      Coordinator notifies Workers about this event. Coordinator and Workers
      maintain a bitmap of executed group that is reset with a new checkpoint. 
   */
-  void reset_notified_checkpoint(ulong, time_t, bool);
+  void reset_notified_checkpoint(ulong, const time_t *const new_ts,
+                                 bool);
 
   /**
      Called when gaps execution is ended so it is crash-safe
@@ -1301,14 +1303,15 @@ private:
 
 public:
   /*
-    The boolean is set to true when the binlog applier (rli_fake) thread
-    detaches any "native" engine transactions it has dealt with
-    at time of XA START processing.
-    The boolean is reset to false at the end of XA PREPARE
-    and XA COMMIT ONE PHASE, at the same time with the native transactions
-    re-attachment.
+    The boolean is set to true when the binlog (rli_fake) or slave
+    (rli_slave) applier thread detaches any engine ha_data
+    it has dealt with at time of XA START processing.
+    The boolean is reset to false at the end of XA PREPARE,
+    XA COMMIT ONE PHASE for the binlog applier, and
+    at internal rollback of the slave applier at the same time with
+    the engine ha_data re-attachment.
   */
-  bool is_native_trx_detached;
+  bool is_engine_ha_data_detached;
 
   void set_thd_tx_priority(int priority)
   {
@@ -1319,6 +1322,30 @@ public:
   {
     return thd_tx_priority;
   }
+  /**
+    Detaches the engine ha_data from THD. The fact
+    is memorized in @c is_engine_ha_detached flag.
+
+    @param  thd a reference to THD
+  */
+  void detach_engine_ha_data(THD *thd);
+  /**
+    Drops the engine ha_data flag when it is up.
+    The method is run at execution points of the engine ha_data
+    re-attachment.
+
+    @return true   when THD has detached the engine ha_data,
+            false  otherwise
+  */
+  bool unflag_detached_engine_ha_data()
+  {
+    bool rc= false;
+
+    if (is_engine_ha_data_detached)
+      rc= !(is_engine_ha_data_detached= false); // return the old value
+
+    return rc;
+  }
 };
 
 bool mysql_show_relaylog_events(THD* thd);
