@@ -677,6 +677,54 @@ void update_global_user_stats(THD* thd, bool create_user, time_t now)
   mysql_mutex_unlock(&LOCK_global_user_client_stats);
 }
 
+static void clear_stats_concurrent_connections(HASH* stats)
+{
+  for (ulong idx= 0; idx < stats->records; idx++)
+  {
+    USER_STATS* const user_stats=
+      reinterpret_cast<USER_STATS*>(my_hash_element(stats, idx));
+    user_stats->concurrent_connections= 0;
+  }
+}
+
+static void inc_stats_concurrent_conn(HASH* stats,
+				      const char* user_string, int cnt)
+{
+  USER_STATS* const user_stats=
+    reinterpret_cast<USER_STATS*>(my_hash_search(stats,
+          reinterpret_cast<const uchar*>(user_string),
+          strlen(user_string)));
+  if (user_stats)
+    user_stats->concurrent_connections+= cnt;
+}
+
+/**
+  Update number of concurrent connections for user_stats and client_stats
+  based on account resource limits
+*/
+void refresh_concurrent_conn_stats()
+{
+  mysql_mutex_lock(&LOCK_user_conn);
+
+  mysql_mutex_lock(&LOCK_global_user_client_stats);
+  clear_stats_concurrent_connections(&global_user_stats);
+  clear_stats_concurrent_connections(&global_client_stats);
+  mysql_mutex_unlock(&LOCK_global_user_client_stats);
+
+  for (ulong idx= 0; idx < hash_user_connections.records; idx++)
+  {
+    const struct user_conn* const uc=
+      reinterpret_cast<struct user_conn*>(
+          my_hash_element(&hash_user_connections, idx));
+    mysql_mutex_lock(&LOCK_global_user_client_stats);
+    inc_stats_concurrent_conn(&global_user_stats, uc->user, uc->connections);
+    inc_stats_concurrent_conn(&global_client_stats, uc->host, uc->connections);
+    mysql_mutex_unlock(&LOCK_global_user_client_stats);
+  }
+
+  mysql_mutex_unlock(&LOCK_user_conn);
+}
+
 /*
   check if user has already too many connections
 
@@ -891,6 +939,10 @@ void release_user_connection(THD *thd)
   DBUG_VOID_RETURN;
 }
 
+void refresh_concurrent_conn_stats()
+{
+}
+
 #endif /* NO_EMBEDDED_ACCESS_CHECKS */
 
 /*
