@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2016, Oracle and/or its affiliates. All Rights Reserved.
+Copyright (c) 1995, 2017, Oracle and/or its affiliates. All Rights Reserved.
 Copyright (c) 2016, Percona Inc. All Rights Reserved.
 
 This program is free software; you can redistribute it and/or modify it under
@@ -429,9 +429,9 @@ static
 void
 buf_parallel_dblwr_close(void)
 {
-	if (parallel_dblwr_buf.file != OS_FILE_CLOSED) {
+	if (!parallel_dblwr_buf.file.is_closed()) {
 		os_file_close(parallel_dblwr_buf.file);
-		parallel_dblwr_buf.file = OS_FILE_CLOSED;
+		parallel_dblwr_buf.file.set_closed();
 	}
 }
 
@@ -450,7 +450,7 @@ recovery, this function loads the pages from double write buffer into memory.
 @return DB_SUCCESS or error code */
 dberr_t
 buf_dblwr_init_or_load_pages(
-	os_file_t	file,
+	pfs_os_file_t	file,
 	const char*	path)
 {
 	byte*		buf;
@@ -464,6 +464,13 @@ buf_dblwr_init_or_load_pages(
 	ibool		reset_space_ids = FALSE;
 	recv_dblwr_t&	recv_dblwr = recv_sys->dblwr;
 
+	if (srv_read_only_mode) {
+
+		ib::info() << "Skipping doublewrite buffer processing due to "
+			"InnoDB running in read only mode";
+		return(DB_SUCCESS);
+	}
+
 	/* We do the file i/o past the buffer pool */
 
 	unaligned_read_buf = static_cast<byte*>(
@@ -616,10 +623,13 @@ buf_dblwr_init_or_load_pages(
 	}
 
 	err = buf_parallel_dblwr_make_path();
-	if (err != DB_SUCCESS)
-		return(err);
+	if (err != DB_SUCCESS) {
 
-	ut_ad(parallel_dblwr_buf.file == OS_FILE_CLOSED);
+		ut_free(unaligned_read_buf);
+		return(err);
+	}
+
+	ut_ad(parallel_dblwr_buf.file.is_closed());
 	bool success;
 	parallel_dblwr_buf.file
 		= os_file_create_simple_no_error_handling(
@@ -648,7 +658,7 @@ buf_dblwr_init_or_load_pages(
 
 		os_file_set_nocache(parallel_dblwr_buf.file,
 				    parallel_dblwr_buf.path,
-				    "open");
+				    "open", false);
 
 		os_offset_t size = os_file_get_size(parallel_dblwr_buf.file);
 
@@ -757,6 +767,8 @@ buf_dblwr_process(void)
 	byte*		unaligned_read_buf;
 	recv_dblwr_t&	recv_dblwr	= recv_sys->dblwr;
 
+	ut_ad(!srv_read_only_mode);
+
 	unaligned_read_buf = static_cast<byte*>(
 		ut_malloc_nokey(2 * UNIV_PAGE_SIZE));
 
@@ -1250,21 +1262,8 @@ buf_dblwr_flush_buffered_writes(
 	srv_stats.dblwr_pages_written.add(dblwr_shard->first_free);
 	srv_stats.dblwr_writes.inc();
 
-	/* Now flush the doublewrite buffer data to disk, unless
-	innodb_flush_method is one of O_SYNC, O_DIRECT_NO_FSYNC, or
-	ALL_O_DIRECT. */
-	switch (srv_unix_file_flush_method) {
-	case SRV_UNIX_NOSYNC:
-	case SRV_UNIX_O_DSYNC:
-	case SRV_UNIX_O_DIRECT_NO_FSYNC:
-	case SRV_UNIX_ALL_O_DIRECT:
-		break;
-	case SRV_UNIX_FSYNC:
-	case SRV_UNIX_LITTLESYNC:
-	case SRV_UNIX_O_DIRECT:
+	if (parallel_dblwr_buf.needs_flush)
 		os_file_flush(parallel_dblwr_buf.file);
-		break;
-	}
 
 	/* We know that the writes have been flushed to disk now
 	and in recovery we will find them in the doublewrite buffer
@@ -1496,7 +1495,7 @@ buf_parallel_dblwr_file_create(void)
 	if (err != DB_SUCCESS)
 		return(err);
 
-	ut_ad(parallel_dblwr_buf.file == OS_FILE_CLOSED);
+	ut_ad(parallel_dblwr_buf.file.is_closed());
 	ut_ad(parallel_dblwr_buf.recovery_buf_unaligned == NULL);
 
 	/* Set O_SYNC if innodb_flush_method == O_DSYNC. */
@@ -1514,20 +1513,27 @@ buf_parallel_dblwr_file_create(void)
 			ib::error() << "A parallel doublewrite file "
 				    << parallel_dblwr_buf.path
 				    << " found on startup.";
-			if (srv_force_recovery == SRV_FORCE_NO_LOG_REDO) {
-				ib::error() << "Since --innodb-force-recovery "
-					"is set to 6, which skips doublewrite "
-					"buffer recovery, please move away "
-					"the file above and restore it before "
-					"attempting a lower forced recovery "
-					"setting";
-			}
 		}
 		return(DB_ERROR);
 	}
 
-	os_file_set_nocache(parallel_dblwr_buf.file, parallel_dblwr_buf.path,
-			    "create");
+	const bool o_direct_set
+		= os_file_set_nocache(parallel_dblwr_buf.file,
+				      parallel_dblwr_buf.path,
+				      "create", false);
+	switch (srv_unix_file_flush_method) {
+	case SRV_UNIX_NOSYNC:
+	case SRV_UNIX_O_DSYNC:
+	case SRV_UNIX_O_DIRECT_NO_FSYNC:
+	case SRV_UNIX_ALL_O_DIRECT:
+		parallel_dblwr_buf.needs_flush = !o_direct_set;
+		break;
+	case SRV_UNIX_FSYNC:
+	case SRV_UNIX_LITTLESYNC:
+	case SRV_UNIX_O_DIRECT:
+		parallel_dblwr_buf.needs_flush = true;
+		break;
+	}
 
 	success = os_file_set_size(parallel_dblwr_buf.path,
 				   parallel_dblwr_buf.file, size, false);
@@ -1550,8 +1556,7 @@ the disk file.
 dberr_t
 buf_parallel_dblwr_create(void)
 {
-	if (parallel_dblwr_buf.file != OS_FILE_CLOSED
-	    || srv_read_only_mode) {
+	if (!parallel_dblwr_buf.file.is_closed() || srv_read_only_mode) {
 
 		ut_ad(parallel_dblwr_buf.recovery_buf_unaligned == NULL);
 		return(DB_SUCCESS);
